import { Box, Button, Checkbox, Code, Group, Modal, NumberInput, Select, Text, Title } from '@mantine/core';
import { useForm } from '@mantine/form';
import { DownloadIcon } from 'components/icons';
import Link from 'components/Link';
import MutedText from 'components/MutedText';
import { useState } from 'react';

export function GeneratorModal({ opened, onClose, title, onSubmit, ...other }) {
  const form = useForm({
    initialValues: {
      type: 'upload-file',
      format: 'RANDOM',
      imageCompression: 0,
      zeroWidthSpace: false,
      embed: false,
      wlCompatibility: false,
      wlCompositorNotSupported: false,
      noJSON: false,
    },
  });

  const [isUploadFile, setIsUploadFile] = useState(true);

  const onChangeType = (value) => {
    setIsUploadFile(value === 'upload-file');
    form.setFieldValue('type', value);
  };

  return (
    <Modal opened={opened} onClose={onClose} title={<Title order={3}>{title}</Title>} size='lg'>
      {other.desc && (
        <MutedText size='md' mb='md'>
          {other.desc}
        </MutedText>
      )}
      <form onSubmit={form.onSubmit((values) => onSubmit(values))}>
        <Select
          label='Type'
          data={[
            { value: 'upload-file', label: 'Upload file' },
            { value: 'shorten-url', label: 'Shorten URLs' },
          ]}
          id='type'
          my='sm'
          {...form.getInputProps('type')}
          onChange={onChangeType}
        />

        <Select
          label='Select file name format'
          data={[
            { value: 'RANDOM', label: 'Random (alphanumeric)' },
            { value: 'DATE', label: 'Date' },
            { value: 'UUID', label: 'UUID' },
            { value: 'NAME', label: 'Name (keeps original file name)' },
          ]}
          id='format'
          my='sm'
          disabled={!isUploadFile}
          {...form.getInputProps('format')}
        />

        <NumberInput
          label='Image Compression'
          description='Set the image compression level (0-100). 0 is no compression, 100 is maximum compression.'
          max={100}
          min={0}
          my='sm'
          id='imageCompression'
          disabled={!isUploadFile}
          {...form.getInputProps('imageCompression')}
        />

        <Group grow my='md'>
          <Checkbox
            label='Zero Width Space'
            description='Use zero width spaces as the file name'
            id='zeroWidthSpace'
            {...form.getInputProps('zeroWidthSpace', { type: 'checkbox' })}
          />
          <Checkbox
            description='Image will display with embedded metadata'
            label='Embed'
            id='embed'
            disabled={!isUploadFile}
            {...form.getInputProps('embed', { type: 'checkbox' })}
          />
          <Checkbox
            description='Return response as plain text instead of JSON'
            label='No JSON'
            id='noJSON'
            {...form.getInputProps('noJSON', { type: 'checkbox' })}
          />
        </Group>

        {title === 'Flameshot' && (
          <>
            <Box my='md'>
              <Text>Wayland</Text>
              <MutedText size='sm'>
                If using wayland, you can check the boxes below to your liking. This will require{' '}
                <Link href='https://github.com/bugaevc/wl-clipboard'>
                  <Code>wl-clipboard</Code>
                </Link>{' '}
                for the <Code>wl-copy</Code> command.
              </MutedText>
            </Box>

            <Group my='md'>
              <Checkbox
                label='Enable Wayland Compatibility'
                description={
                  <>
                    Use <Code>wl-copy</Code> instead of <Code>xsel -ib</Code>
                  </>
                }
                id='wlCompatibility'
                {...form.getInputProps('wlCompatibility', { type: 'checkbox' })}
              />

              <Checkbox
                label={
                  <>
                    Using a DE/compositor that <b>isn&apos;t</b> GNOME, KDE or Sway
                  </>
                }
                description={
                  <>
                    If using a compositor such as{' '}
                    <Link href='https://github.com/hyprwm/hyprland'>Hyprland</Link>, this option will set the{' '}
                    <Code>XDG_CURRENT_DESKTOP=sway</Code> to workaround Flameshot&apos;s errors
                  </>
                }
                disabled={!isUploadFile}
                id='wlCompositorNotSupported'
                {...form.getInputProps('wlCompositorNotSupported', { type: 'checkbox' })}
              />
            </Group>
          </>
        )}

        <Group grow my='md'>
          <Button onClick={form.reset}>Reset</Button>

          <Button rightIcon={<DownloadIcon />} type='submit'>
            Download
          </Button>
        </Group>
      </form>
    </Modal>
  );
}
